// Customized version of lftkryo's Chip Tune Player - ISR Only
// 17-Oct-2024 by KPK
// Video: https://www.youtube.com/watch?v=ly5BhGOt2vE
// Assembler: Kick

.const new_border = $02 // border color that is set in the note_on section

* = $c000

setup:
  lda $314     // check whether isr is already active
  cmp #<isr
  bne !+
  lda $315
  cmp #>isr
  bne !+
  rts
!: jsr init_sid // initialize sid chip
  lda #$7f
  sta $dc0d    // disable timer interrupts
  lda #<isr    // new isr vector
  sta $314
  lda #>isr
  sta $315
  lda #$01
  sta $d01a    // enable raster interrupts
  sta $d012    // set interrupt for scanline 1
  rts

isr:
  lda new_border // border color that is set in the note_on section
  sta $d020    // set border color
  lda #$01
  sta $d019    // acknowledge raster interrupt
  lda $a2      // get jiffy clock least-significant-byte
  lsr          // divide it by 8 (256 / 8 = 32 possible values)
  lsr
  lsr
  tay          // transfer a to y to use it as an index/offset
  lda #$0e     // light blue (default) color
  sta $d856,y  // color ram (2 rows down, 6 chars right) (measure marker)
  sta $d876    // right-most column char
  lda #$01     // white
  sta $d857,y  // color ram (2 rows down, 7 chars right) (measure marker)
  lda $47f,y   // screen ram (3 rows down, 7 chars right) (voice 1)
  pha          // push to stack
  lda $4a7,y   // screen ram (4 rows down, 7 chars right) (voice 2)
  pha          // push to stack
  lda $4cf,y   // screen ram (5 rows down, 7 chars right) (voice 3)
  ldx #$0e     // sid register offset for voice 3
  jsr note_on  // turn on note
  pla          // pull from stack
  ldx #$07     // sid register offset for voice 2
  jsr note_on  // turn on note
  pla          // pull from stack
  ldx #$00     // sid register offset for voice 1
  jsr note_on  // turn on note
  lda #$0e     // light blue (default) color
  sta $d020    // set border color
  jmp $ea31    // continue to standard interrupt vector

note_on:
  cmp #$20     // space character
  bcc !+       // check whether character code < 32
  jmp note_off // if not, turn off note
!: cpx #$00     // check for voice 1
  bne !+       // skip for voice 2 or 3
  sta new_border // use the note value later for the border color in the raster interrupt
!: adc sid_params,x // if so, add pitch semitone offset for selected voice
  tay          // make this an index into the pitch table
  lda pitch_table_lo,y // get pitch lo byte
  sta $d400,x  // set pitch lo byte for selected voice
  lda pitch_table_hi,y // get pitch hi byte
  sta $d401,x  // set pitch hi byte for selected voice
  lda sid_params+1,x // get waveform gate on parameter for selected voice
  sta $d404,x  // set waveform gate on parameter for selected voice
  rts

note_off:
  lda sid_params+2,x // get waveform gate off parameter for selected voice
  sta $d404,x  // set waveform gate off parameter for selected voice
  bpl !+       // skip if noise waveform is selected
  sta $d401,x  // otherwise set it for the selected voice (for percussion effect)
!: rts

init_sid:      // initialize sid registers
  ldx #24
!: lda sid_startup,x
  sta $d400,x
  dex
  bpl !-
  rts

sid_startup:
  // initialization table for sid registers - AD/SR/Pulse Width/Volume
  .byte 0,0,0,0,0,0,239 // voice 1
  .byte 0,0,0,4,0,0,224 // voice 2
  .byte 0,0,0,0,0,128,232 // voice 3
  .byte 0,0,0,15 // volume

sid_params:
  // parameters - pitch table offset, voice waveform gate on, voice waveform gate off
  // followed by 4 bytes of padding for alignment
  .byte 36,17,16,0,0,0,0
  .byte 24,65,64,0,0,0,0
  .byte 0,33,128,0,0,0,0

pitch_table_lo:
  .byte $3a,$5b,$7f,$a5,$ce,$f8,$26,$56,$88,$be,$f7,$34,$74,$b7,$ff,$4b
  .byte $9c,$f1,$4c,$ac,$11,$7d,$ef,$68,$e8,$6f,$ff,$97,$38,$e3,$98,$58
  .byte $23,$fa,$de,$d0,$d0,$df,$fe,$2e,$71,$c6,$30,$b0,$46,$f4,$bc,$a0
  .byte $a0,$be,$fc,$5d,$e2,$8d,$61,$60,$8d,$e9,$79,$40,$40,$7c,$f9,$bb
  .byte $c4,$1b,$c3,$c1,$1a

pitch_table_hi:
  .byte $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04,$04,$04,$04,$05
  .byte $05,$05,$06,$06,$07,$07,$07,$08,$08,$09,$09,$0a,$0b,$0b,$0c,$0d
  .byte $0e,$0e,$0f,$10,$11,$12,$13,$15,$16,$17,$19,$1a,$1c,$1d,$1f,$21
  .byte $23,$25,$27,$2a,$2c,$2f,$32,$35,$38,$3b,$3f,$43,$47,$4b,$4f,$54
  .byte $59,$5f,$64,$6a,$71