; Customized version of lftkryo's Chip Tune Player - ISR and Program Listing
; 17-Oct-2024 by KPK
; Video: https://www.youtube.com/watch?v=ly5BhGOt2vE
; Assembler: Acme

!addr new_border = $02 ; border color that is set in the note_on section
!addr zp_src     = $fb ; zero page pointer to source (program) data
!addr zp_dst     = $fd ; zero page pointer to destination (screen)

* = $c000

setup:
  lda $314     ; check whether isr is already active
  cmp #<isr
  bne +
  lda $315
  cmp #>isr
  bne +
  rts
+ jsr init_sid ; initialize sid chip
  jsr show_prg ; show program listing
  lda #$7f
  sta $dc0d    ; disable timer interrupts
  lda #<isr    ; new isr vector
  sta $314
  lda #>isr
  sta $315
  lda #$01
  sta $d01a    ; enable raster interrupts
  sta $d012    ; set interrupt for scanline 1
  rts

isr:
  lda new_border ; border color that is set in the note_on section
  sta $d020    ; set border color
  lda #$01
  sta $d019    ; acknowledge raster interrupt
  lda $a2      ; get jiffy clock least-significant-byte
  lsr          ; divide it by 8 (256 / 8 = 32 possible values)
  lsr
  lsr
  tay          ; transfer a to y to use it as an index/offset
  lda #$0e     ; light blue (default) color
  sta $d856,y  ; color ram (2 rows down, 6 chars right) (measure marker)
  sta $d876    ; right-most column char
  lda #$01     ; white
  sta $d857,y  ; color ram (2 rows down, 7 chars right) (measure marker)
  lda $47f,y   ; screen ram (3 rows down, 7 chars right) (voice 1)
  pha          ; push to stack
  lda $4a7,y   ; screen ram (4 rows down, 7 chars right) (voice 2)
  pha          ; push to stack
  lda $4cf,y   ; screen ram (5 rows down, 7 chars right) (voice 3)
  ldx #$0e     ; sid register offset for voice 3
  jsr note_on  ; turn on note
  pla          ; pull from stack
  ldx #$07     ; sid register offset for voice 2
  jsr note_on  ; turn on note
  pla          ; pull from stack
  ldx #$00     ; sid register offset for voice 1
  jsr note_on  ; turn on note
  lda #$0e     ; light blue (default) color
  sta $d020    ; set border color
  jmp $ea31    ; continue to standard interrupt vector

note_on:
  cmp #$20     ; space character
  bcc +        ; check whether character code < 32
  jmp note_off ; if not, turn off note
+ cpx #$00     ; check for voice 1
  bne +        ; skip for voice 2 or 3
  sta new_border ; use the note value later for the border color in the raster interrupt
+ adc sid_params,x ; if so, add pitch semitone offset for selected voice
  tay          ; make this an index into the pitch table
  lda pitch_table_lo,y ; get pitch lo byte
  sta $d400,x  ; set pitch lo byte for selected voice
  lda pitch_table_hi,y ; get pitch hi byte
  sta $d401,x  ; set pitch hi byte for selected voice
  lda sid_params+1,x ; get waveform gate on parameter for selected voice
  sta $d404,x  ; set waveform gate on parameter for selected voice
  rts

note_off:
  lda sid_params+2,x ; get waveform gate off parameter for selected voice
  sta $d404,x  ; set waveform gate off parameter for selected voice
  bpl +        ; skip if noise waveform is selected
  sta $d401,x  ; otherwise set it for the selected voice (for percussion effect)
+ rts

init_sid:      ; initialize sid registers
  ldx #24
- lda sid_startup,x
  sta $d400,x
  dex
  bpl -
  rts

show_prg:      ; display sample program listing
  lda $801     ; unless a program is already loaded
  bne +
  lda $802
  beq ++
+ rts
++
  lda #$93     ; clear screen
  jsr $ffd2
  lda #$13     ; home cursor
  jsr $ffd2
  lda #$00     ; set screen as destination
  sta zp_dst
  lda #$04
  sta zp_dst+1
  lda #<program_listing ; set program listing as source
  sta zp_src
  lda #>program_listing
  sta zp_src+1
  ldx #$02     ; prepare to copy two pages of screen data
  ldy #$00
- lda (zp_src),y
  sta (zp_dst),y
  iny
  bne -
  inc zp_src+1
  inc zp_dst+1
  dex
  bne -
  ldx #$00
  lda #$0d
- sta $277,x   ; stuff keyboard buffer with carraige returns
  inx
  cpx #$0a
  bne -
  stx $c6      ; set count of characters in keyboard buffer
  rts

sid_startup:
  ; initialization table for sid registers - AD/SR/Pulse Width/Volume
  !byte 0,0,0,0,0,0,239 ; voice 1
  !byte 0,0,0,4,0,0,224 ; voice 2
  !byte 0,0,0,0,0,128,232 ; voice 3
  !byte 0,0,0,15 ; volume

sid_params:
  ; parameters - pitch table offset, voice waveform gate on, voice waveform gate off
  ; followed by 4 bytes of padding for alignment
  !byte 36,17,16,0,0,0,0
  !byte 24,65,64,0,0,0,0
  !byte 0,33,128,0,0,0,0

pitch_table_lo:
  !hex 3a5b7fa5cef8265688bef73474b7ff4b9cf14cac117def68e86fff9738e3985823fade
  !hex d0d0dffe2e71c630b046f4bca0a0befc5de28d61608de97940407cf9bbc41bc3c11a

pitch_table_hi:
  !hex 0202020202020303030303040404040505050606070707080809090a0b0b0c0d0e0e0f
  !hex 10111213151617191a1c1d1f212325272a2c2f3235383b3f43474b4f54595f646a71

program_listing:
  !hex 2020202020202020202020202020202020202020202020202020202020202020202020
  !hex 2020202020202020202020202020202020202020202020202020202020202020202020
  !hex 202020202020202020203430201012090e14030812242831343729220c09131438302d
  !hex 38392220202020202020202020202038302012050d202b2d2d2d2b2d2d2d2b2d2d2d2b
  !hex 2d2d2d2b2d2d2d2b2d2d2d2b2d2d2d2b2d2d2d2038312012050d200d2020203a202020
  !hex 3a20202014202020112020203a2020203a2020200f2020202038322012050d20142016
  !hex 143a12110d0f201420181614113a0d2019181614113a0d201411201220203833201205
  !hex 0d20010d20010d01200d08142008140820140a16200a160a2016061220061206201220
  !hex 38342012050d2020202020202020202020202020202020202020202020202020202020
  !hex 202020202038352012050d202020022004202020072009200b2020200e201020202013
  !hex 201520172020201a202038362012050d2020012003200520062008200a200c200d200f
  !hex 2011201220142016201820192020203930201319133139322a3235363a0c0913143830
  !hex 2d3839202020202020202020202020202020202012150e202020202020202020202020
  !hex 20202020202020202020202020202020202020202020202020a0202020202020202020
  !hex 20202020202020202020202020202020202020202020